#!/bin/bash
VER=2.5.3
#---------------------------------------------------------------------------#
#                                                                           #
# Tur-Search.                                                               #
# Two small scripts that allow users to do like 'site search' from glftpd   #
# and !search from irc.                                                     #
#                                                                           #
# Yes, glftpd already supports 'site search' but this one has a few tricks. #
# It shows if the release exists on the site still (might be in dirlog but  #
# not on site) and if so, the total size of it.                             #
#                                                                           #
# It also has the ability to create symlinks to each release it finds in a  #
# place you decide, so if the user wants to download a lot of stuff, he/she #
# does not have to read the text and manually enter all dirs. Just tag all  #
# the symlinks and off we go.                                               #
#                                                                           #
# Specifying words that should be excluded from the results are also        #
# possible.                                                                 #
#                                                                           #
# Combine this with Tur-DirLogClean to keep the dirlog nicely updated!!     #
#                                                                           #
#---------------------------------------------------------------------------#
# Installation:                                                             #
#                                                                           #
# Put tur-search in /glftpd/bin                                             #
#                                                                           #
# Put tur-search.tcl in your bots scripts folder and load it in the bots    #
# config file. Rehash the bot and make sure it says that its loading it.    #
#                                                                           #
# Make tur-search.sh executable with chmod 755 tur-search.sh                #
#                                                                           #
# Change the settings below:                                                #
#                                                                           #
# glroot=     The path to your glftpd dir. Usually just /glftpd             #
#                                                                           #
# dirloglistirc= The path to the dirloglist binary. This comes with glftpd  #
#                and should already exist.                                  #
#                It will be used when searching from irc or shell.          #
#                Make sure it works by running it from shell.               #
#                                                                           #
# dirloglistgl=  Since 'dirloglist' does not work from inside glftpd, I had #
#                to make a dirty hack in the source for it so that it does. #
#                Since I cant code that stuff and only changed the path, it #
#                is a seperate binary. The binaries are included and are    #
#                compiled on Fedora Core 3. The sources are included as     #
#                well if you need to compile them yourself.                 #
#                                                                           #
#                They're in the sources dir and compiles as:                #
#                gcc dirloglist_gl -o /glftpd/bin/dirloglist_gl             #
#                                                                           #
#                Important, if your glftpd.conf is not in /etc and you use  #
#                glftpd 2+, you MUST edit glconf.h in the glftpd2 dir and   #
#                specify the path to glftpd.conf before compiling           #
#                dirloglist_gl.c                                            #
#                                                                           #
# maxhitsirc= Maximum number of hits to return when running from irc/shell. #
# maxhitsgl=  Maximum number of hits to return when running from glftpd.    #
#                                                                           #
# minlenght=  Minimum lenght to search for when only searching using one    #
#             argument. Set to "" to disable (not recomended).              #
#                                                                           #
# ignoredirs= Ignore these words in search. Good to add the name of the pre #
#             dirs here. Use /PRE to prevent anything starting with PRE to  #
#             be displayed. Use /PRE/ to match the exact foldername.        #
#             Predirs should not be in the dirlog anyway, but better safe.. #
#             Seperate ignores with a |  ( '/GROUPS/|Animalporn|ost' )      #
#             Any weird chars should be marked with \, ie \[inco            #
#                                                                           #
#             Use a . to define * , ie: CD.                                 #
#             Those familiar with grep can of course use CD[1-9] as well.   #
#                                                                           #
# showsize=   If this is TRUE, it will run 'du' on the dir it finds and     #
#             reports its size for every hit.                               #
#                                                                           #
# du_options= These are the options that are used for du to get the size.   #
#             Note on this. If you search and get errors such as as:        #
#             du: /path/to/rel/[NUKED...]: Permission denied                #
#             then check 'man du' and find the options for your system on   #
#             how to disable checking those dirs. For FBSD, something like  #
#             -I "[NUKED*" or just -I "[*" would do it.                     #
#             For RedHat and most linuxes, its --exclude="[NUKED*"          #
#             Just add those options in du_options                          #
#                                                                           #
# sitedir=    If the above is TRUE, what is your sitedir? /site as          #
#             default. When doing a search, check the path. This is what    #
#             needs to be before the path for 'du' to find it.              #
#                                                                           #
# symsdir=    If you have it loaded as 'site search' in glftpd and want the #
#             users to be able to create symlinks to their findings, set    #
#             this to a directory inside your /site structure somewhere.    #
#             Once a user specifies --create in their search string, it     #
#             will create a directory with their name in here and put a     #
#             symlink to each hit they get. This is why it only works from  #
#             glftpd. We have to sort them somehow and the username is the  #
#             easiest way. Cant have everyone create symlinks in the same   #
#             directory or it would get ... messy.                          #
#                                                                           #
#             Before starting the search, their own "symlinkdir" will be    #
#             cleaned out, using the unlink binary.                         #
#                                                                           #
#             The symlinks are cleaned out when the user does a new search  #
#             (if they do not use --noclear as well), but their dir is      #
#             not. If might therefor be a nice idea to clear out the        #
#             symsdir once a day or similar.                                #
#             A simple crontab can take care of that. Something like:       #
#             0 0 * * * rm -rf /path/to/symsdir/* >/dev/null 2>&1           #
#             should do fine.                                               #
#                                                                           #
#             Oh yeah, create this dir in shell and set chmod 777 on it.    #
#                                                                           #
#                                                                           #
# Now then, if you wanted to replace it with your current 'site search'     #
# command in glftpd, add this to glftpd.conf:                               #
# site_cmd search EXEC /bin/tur-search.sh                                   #
# custom-search *                                                           #
# Of course, you could also use a different name for it. Just replace       #
# search with whatever you want to call it with.                            #
#                                                                           #
# Important: This script requires the 'unlink' binary to be present in      #
# /glftpd/bin if you want to load it as 'site search' in glftpd and have    #
# symsdir set. I dont trust rm -f to not remove the actual release.         #
# It also requires some standard ones like:                                 #
# grep, egrep, cut, tr, du, sed & tail (dirloglist and dirloglist_gl too)   #
#                                                                           #
# Run tur-search.sh from shell and make sure it works.                      #
# When you test it from the bot you might get an error like "permission     #
# denied /root/.bashrc".                                                    #
# Check the (general) FAQ on the website on how to fix that.                #
#                                                                           #
# For instructions in both irc and glftpd, just run it without any args.    #
#                                                                           #
#---------------------------------------------------------------------------#
# http://www.grandis.nu/glftpd/                                             #
#---------------------------------------------------------------------------#
# Changelog:                                                                #
#                                                                           #
# 2.5.3 : Fixed a problem with --create --noclear when it found a release   #
#         there already was a symlink for, it would create the symlink in   #
#         the release itself, pointing to itself (bad).                     #
#         Now, WHY nobody detected this in more then a year makes me think  #
#         this function is not really used. In either case, its now fixed.  #
#                                                                           #
#         Replace tur-search.sh if upgrading.                               #
#                                                                           #
# 2.5.2 : Fixed problem when searching for stuff that includes ( ).         #
#         No results would be shown.                                        #
#                                                                           #
# 2.5.1 : The helptext inside glftpd did not display the ! for exclude.     #
#         This has been fixed.                                              #
#                                                                           #
# 2.5   : You can now specify ! at the start of a word to exclude any hit   #
#         including that word. Specify more words with !word1~word2~word3   #
#         and neither of those words can be in the full release path/name   #
#         for it to be considered a hit.                                    #
#                                                                           #
#         Fixed a minor issue when using --create. If you got two hits and  #
#         it created a symlink to some.movie.part2 and it also found        #
#         some.movie, it would think it already created that symlink.       #
#                                                                           #
# 2.4   : Added a dirloglist_gl.c for glftpd 2+ in the sources dir.         #
#                                                                           #
# 2.3   : Ok, rewritten to use dirloglist instead of strings since that was #
#         a pain. See installation instructions at the top.                 #
#                                                                           #
# 2.2   : Now working for FBSD as well with the help from [PD]Raky          #
#         Thanks!                                                           #
#                                                                           #
# 2.1   : When checking diskusage (if showsize=true) and it hit a nuked dir #
#         you got a permission denied error - fixed.                        #
#                                                                           #
#         If it hit an entry in the dirlog with a tab in it, it would halt  #
#         - fixed.                                                          #
#-                                                                         -#
# 2.0   : Pretty much rewritten from scratch.                               #
#                                                                           #
#         * Renamed from tur-botsearch to tur-search since it now works     #
#           from inside glftpd as well.                                     #
#                                                                           #
#         * Changed how the search works. You can now specify how many      #
#           arguments as you want to search. It should also be quite a bit  #
#           faster.                                                         #
#           The drawback to this is that the order of the words must match  #
#           the order they appear in the release.                           #
#           I guess some people see this as a good thing, as long as you    #
#           know about it.                                                  #
#           Its described in the help for the script.                       #
#                                                                           #
#         * Added a neat option. If you load this as 'site search' in       #
#           glftpd, you can have it create symlinks to each result it gets  #
#           into a predefined directory. This is GREAT when searcing for    #
#           lots of stuff you want to download as the user can just go into #
#           the symlinkdir and tag the releases for download without having #
#           to manually enter each dir, go one step up and then download    #
#           it one by one.                                                  #
#                                                                           #
#           Make sure you read through the installation instructions again  #
#           as it changed quite a bit as well as running it without args    #
#           to get usage instructions on --create and --noclear             #
#                                                                           #
#           The above idea was suggested by m0n. Good one =)                #
#                                                                           #
# People upgrading from old tur-botsearch will need to replace the tcl as   #
# well. Notice that its tur-search.tcl instead of tur-botsearch.tcl so edit #
# your bots config file to reflect that and dont forget to .rehash          #
#                                                                           #
#-                                                                         -#
# 1.4   : Will work with releases and sections with spaces in them now.     #
#         Will use ~ as a tempchar for space, so if you have dirs with ~ in #
#         them, they will be a space in search. Just search for '~' and     #
#         change that to some other char you dont use if you dont like it.  #
#         Thanks liquid- for testing it for me.                             #
#                                                                           #
# 1.3   : When using 3 arguments, it first searched on 2, then again with   #
#         3, making it take double the time to finish.                      #
#                                                                           #
#         Added options showsize and sitedir. Check explanation above.      #
#         This will return MB value of release. If it is not found, it will #
#         say '-> Deleted?'. If it was found, but couldnt check size of it, #
#         size will be set to '?'.                                          #
#         There are some explanation texts down in the script if you want   #
#         to change the text.                                               #
#                                                                           #
#         Expanded it to allow 5 search words instead of just 3. REPLACE    #
#         the TCL if you want to use this.                                  #
#                                                                           #
#         In 'ignoredirs' you can use '.' as '*'                            #
#         So... replaced all CD1 etc with CD.                               #
#                                                                           #
#         A oneliner telling the user what he searched on is echoed as a    #
#         header for each search.                                           #
#                                                                           #
#         Added check that first arg is not a *. Could hang the bot.        #
#                                                                           #
# 1.2   : Search may now start with -                                       #
# 1.1   : Ignoredirs are no longer case sensitive. Also added a few extra   #
#         to the default so you wont get those pesky CD1 CD2 etc etc.       #
#         Remember to escape any non "normal" chars (whatever their names   #
#         are). IE [incomplete] = \[incomplete\]                            #
#    Upg: change all lines with contains:                                   #
#         egrep -v $ignoredirs to egrep -vi $ignoredirs                     #
#         Paste the new ignoredirs into your old script.                    #
#         Update the version at the top to 1.1 =))                          #
#                                                                           #
# 1.0   : First public version.                                             #
#---------------------------------------------------------------------------#

glroot="/glftpd"

dirloglistirc="/bin/dirloglist"
dirloglistgl="/bin/dirloglist_gl"

maxhitsirc="15"
maxhitsgl="500"

minlenght="4"
ignoredirs="\/PRE\/|\/GROUPS\/|\/CD[1-9]|\/Sample|\[incomplete\]"

showsize="TRUE"
du_options="-hc"

sitedir="/site"

symsdir="/site/Search_Links"

#############################################################################
# Change everything below here. Naaah kidding. Dont change a thing.         #
#############################################################################

## In glftpd or not ?
if [ "$FLAGS" ]; then
  mode="gl"
  strings="$stringsgl"  
  unlinkbin="/bin/unlink"
  maxhits="$maxhitsgl"
  dirloglist="$dirloglistgl"
  helptext="!!"
else

  mode="irc"
  strings="$stringsirc"
  maxhits="$maxhitsirc"
  dirlog="$glroot$dirlog"
  sitedir="$glroot$sitedir"
  dirloglist="$glroot$dirloglistirc"
  helptext="!"
fi

## Show help if no arguments.
if [ -z "$1" ] || [ "$1" = "help" ] && [ -z "$2" ]; then
  echo "Tur-Search $VER by Turranius. Used for searching on the site."
  echo ""
  echo "Usage: Enter a whole release or part of it. When searching for more"
  echo "then one word, make sure that the order is correct."
  echo "Searching for 'active directory' will show releases containing"
  echo "'Bla.Bla.Active.Directory-' but not 'Bla.Bla.Directory.Active-'"
  echo ""
  echo "As first argument, you can select part of the path it should be in."
  echo "For example, searching on '0DAY Active Directory' will only return hits"
  echo "if the path it is in contains the word 0DAY."
  echo ""
  echo "You may also specify an exclude if there are certain words you do NOT"
  echo "want included in the search. Use ${helptext}123 and you'll get no hits on"
  echo "releases including \"123\". To specify more excludes, specify"
  echo "${helptext}123~abc~ost and it will exclude on either 123, abc or ost."
  echo "As with the search, you can specify part of the section/path here to"
  echo "exclude everything in that path."

  if [ "$symsdir" ] && [ "$mode" = "gl" ]; then
    echo ""
    echo "If you want to create symlinks (shortcuts) to each release you find"
    echo "then also enter --create as one of your search arguments."
    echo "Once the search has been completed, it will tell you where your"
    echo "symlinks are for easy downloading."
    echo ""
    echo "Each time you do a new search, the symlinks will be cleared unless you"
    echo "also specify --noclear as an argument. All \"old\" symlinks will then"
    echo "remain between searches."
  fi

  echo ""
  echo "Only $maxhits hits will be shown."
  exit 0
fi

## Set this one incase we destroy it (we will from irc).
symsdirorg="$symsdir"

## Check that first arg isnt *. Probably not needed but cant hurt.
if [ "$1" = '*' ]; then
  echo "No go on that on bud."
  exit 0
fi

## Check if strings exists
if [ ! -x $dirloglist ]; then
  echo "$dirloglist is not executable or does not exist."
  exit 0
fi

FULL_COMMAND=" $@ "

if [ "`echo "$FULL_COMMAND" | grep -i "\ --create\ "`" ]; then
  if [ "`echo "$FULL_COMMAND" | grep -i "\ --noclear\ "`" ]; then
    NOCLEAR="TRUE"
  fi
  if [ "$mode" = "irc" ]; then
    echo "Error: --create only works from inside glftpd. Aborting."
    exit 0
  fi
  if [ "$symsdir" ]; then
    if [ ! -d "$symsdir" ]; then
      echo "Symlink dir $symsdir does not exist. Skipping creation of symlinks."
    else
      if [ ! -e "$unlinkbin" ]; then
        echo "Error. Cant create symlinks because the binary 'unlink' does not exist."
        echo "Make sure it exists in glftpd /bin dir and is executable by all."
      else
        CREATE_LINKS="TRUE"
        if [ -d "$symsdir/$USER" ]; then
          if [ "$NOCLEAR" = "TRUE" ]; then
            echo "Creating symlinks for found releases. --noclean specified so not removing old ones."
          else
            echo "Creating symlinks for found releases. Trying to clear out any old ones."
            echo "Use --noclear as well to not remove previous symlinks."
            echo ""
            for oldsym in `ls -1 "$symsdir/$USER"`; do
              unlink "$symsdir/$USER/$oldsym"
            done
          fi
        fi
      fi
    fi
  else
    echo "Creation of symlinks is not enabled. Skipping --create"
  fi
  FULL_COMMAND="`echo "$FULL_COMMAND" | sed -e 's/\-\-[cC][rR][eE][aA][tT][eE]//g' | sed -e 's/\-\-[nN][oO][cC][lL][eE][aA][rR]//g'`"
else
  if [ "$mode" = "gl" ] && [ "$symsdir" ]; then
    echo ""
    echo "Note that you can use --create somewhere in your search and symlinks (shortcuts)"
    echo "will be created to each hit you get, for easy downloading."
    echo "Run again without any arguments for full help."
    echo ""
  fi
fi    

if [ "`echo "$FULL_COMMAND" | grep -i "\ --noclear\ "`" ]; then
  echo "Error. Cant use --noclear without --create"
  exit 0
fi

## Clean it up a bit (remove excess spaces).
FULL_COMMAND="`echo $FULL_COMMAND | tr -s ' '`"

## Make sure grep line based on arguments.
for command in $FULL_COMMAND; do
  if [ "`echo "$command" | grep "^!."`" ]; then
    command="`echo "$command" | cut -c2-`"
    if [ -z "$SKIP_STRING" ]; then
      SKIP_STRING="$command"
      SKIP_TEXT=", NOT including $SKIP_STRING"
    else
      echo "Error. Only one \"!word\" can be used. Use !word1~word2~word3 to specify more excludes."
      exit 0
    fi
  else
    if [ -z "$SEARCH_STRING" ]; then
      first_command="$command"
      SEARCH_STRING="$command"
      TEXT="containing $command"
    else
      TEXT="$TEXT & $command"
      last_command="$command"
      SEARCH_STRING="$SEARCH_STRING.*$command"
    fi
  fi
done

## Some debug stuff
# echo "Full command: $FULL_COMMAND"
# echo "Create_links: $CREATE_LINKS"
# echo "Search string: $SEARCH_STRING"
# echo "Mode: $mode"
# echo "-------------"

if [ -z "$first_command" ]; then
  echo "Error. You did not search for anything."
  exit 0
fi

## Complain if lenght is too short (size dosnt matter!).
if [ "$minlenght" ] && [ -z "$last_command" ]; then
  tempnum=1
  lenght="."

  while [ "$tempnum" -lt "$minlenght" ]; do
    tempnum=$[$tempnum+1]
    lenght=".$lenght"
  done
      
  if [ -z "`echo "$first_command" | grep "$lenght"`" ]; then
    echo "Search must be a minumum of $minlenght chars with only one argument."
    exit 0
  fi
fi

## Make a lame fix incase ignoredirs are empty.
if [ "$ignoredirs" = "" ]; then
  ignoredirs="kJEFKLejJd3jd"
fi

## Echo header.
if [ "$last_command" ]; then
  echo "Starting search for releases $TEXT (in that order)$SKIP_TEXT" | sed -s 's/~/\ or\ /g'
else
  echo "Starting search for releases $TEXT$SKIP_TEXT" | sed -s 's/~/\ or\ /g'
fi

num="0"

## Fix if theres no excludes defined, otherwise, replace ~ with | for the egrep.
if [ -z "$SKIP_STRING" ]; then
  SKIP_STRING="RFJK567"
else
  SKIP_STRING="`echo "$SKIP_STRING" | sed -e 's/~/\|/g'`"
fi

for each in `$dirloglist | cut -d ':' -f3 | grep -i -- "$SEARCH_STRING" | egrep -vi -- "$SKIP_STRING" | cut -c2- | egrep -vi "$ignoredirs" | tr ' ' '~' | tail -n $maxhits`; do
  gotone="yes"
  num=$[$num+1]
  if [ "$mode" = "irc" ]; then
    each="$glroot$each"
  fi

  ## Make a nice ANNOUNCEREL without /glftpd/site
  tempsitedir="`echo "$sitedir" | tr '/' '+'`"
  tempeach="`echo "$each" | tr '/' '+'`"
  ANNOUNCEREL="`echo "$tempeach" | sed -e "s/$tempsitedir//" | tr '+' '/' `"

  if [ "$showsize" = "TRUE" ]; then
    checkdir="`echo "$each" | tr '~' ' '`"
    if [ ! -e "$checkdir" ]; then

      ## If the dir was not found at all, say this.
      echo "$ANNOUNCEREL -> Deleted?" | tr '~' ' '

    else
      unset size

      size=`du $du_options "$each" | cut -f1 | tail -n1`

      if [ -z "$size" ]; then
        ## IF size was not found for some reason, size will be set to this.
        size="?"
      fi

      ## This is what it says when reporting size.
      echo "$ANNOUNCEREL -> $size" | tr '~' ' '
      SYMLINK="/site$ANNOUNCEREL"
      RELEASENAME="`basename $each`"
    fi

  else

    ## showsize is not TRUE. Just echo release path.
    echo "$ANNOUNCEREL" | tr '~' ' '
    SYMLINK="/site$ANNOUNCEREL"
    RELEASENAME="`basename $each`"
  fi

  ## Create symlink
  if [ "$CREATE_LINKS" = "TRUE" ] && [ "$SYMLINK" ]; then
    if [ ! -d "$symsdir/$USER" ]; then
      mkdir -m777 "$symsdir/$USER"
    fi
    if [ ! -e "$symsdir/$USER/$RELEASENAME" ]; then
      ln -f -s "$SYMLINK" "$symsdir/$USER/$RELEASENAME"
    else
      echo "-- Symlink already exists for $RELEASENAME"
    fi
    unset SYMLINK
    SYM_CREATED="TRUE"
  fi

  ## If the user reached maximum allowed hits.
  if [ "$num" = "$maxhits" ]; then
    MAX="yes"
    break
  fi
done


if [ "$SYM_CREATED" ] && [ "$gotone" = "yes" ]; then
  symnice="`echo "$symsdirorg" | sed -e 's/\/site//'`"
  echo ""
  echo "-- You will find symlinks to your findings in $symnice/$USER"
  echo ""
fi

## No hits? Say this.
if [ "$gotone" != "yes" ];then
  echo "0 Results Returned"
  if [ "$last_command" ]; then
    echo "NOTE: The search words must be in the same order as they appear in the release."
  fi
else

  ## If only one hit was found.
  if [ "$num" = "1" ]; then
    echo "$num hit found."

  ## If more then one hit was found.
  else
    if [ "$MAX" = "yes" ]; then
      echo "Maximum $maxhits returned. Be more specific!."
    else
      echo "$num hits found."
    fi
  fi
fi

exit 0
